;+**********************************************************************
; SCCS id: %W% %U% %G% 
;***********************************************************************
;
; NAME: GPOLROT
;
; PURPOSE:
; This function defines an cartesian trafomatrix from system S to
; a system T defined by a polar lat and ascending node longitude and
; the angle between prime meridian and ascending node.
; Eulerian rotation (see Madelung p.157).
; for planetary systems see Astr.Almanac sect.L
;	
; CATEGORY: Coordinate transformation
;
; CALLING SEQUENCE: emat = GPOLROT(phi,theta,w0)
;
; INPUTS:
; 	      phi   dblarr(npoints) longitude of ascending node    [rad]
;                    note that the pole meridian is given by phi-pi/2
;             theta dblarr(npoints) latitude of pole (zaxis=0)     [rad]
;             w0    dblarr(npoints) angle betw. prime mer&asc node [rad]
;
; OUTPUTS:  emat  dblarr(3,3,npoints) cartesian right handed matrix
;                   if Xvec0 is a column vector in system S, its coordinates
;                   in system T are given by Xvec=Xvec0#emat
;                   unity matrix for pole.phi=-pi/2 !!!
;                   Note that INVERT(emat)=TRANSPOSE(emat) (orthogonal)
;
; PROCEDURE: See Madelung p.157.
;	     For planetary systems see Astr.Almanac sect.L
;
; RESTRICTIONS: The three input parameters must either be arrays
;           of the same size or they might have one element only.
; COMMENT towards Hapgood, Plan.Space Sci.,40,711,1992:
;         The three principal rotations are given by:
;         <zeta,X> = GPOLROT(0,zeta,0)
;         <zeta,Y> = GPOLROT(pi/2,zeta,-pi/2)
;         <zeta,Z> = GPOLROT(0,0,zeta)
;
; MODIFICATION HISTORY:
; 	Written by:M.Fraenz feb 00
;      
;
;-
  FUNCTION GPOLROT,phi,theta,w0
  nph=N_ELEMENTS(phi)
  nth=N_ELEMENTS(theta)
  nw=N_ELEMENTS(w0)
  nel=MAX([nph,nth,nw])
  ematrix=DBLARR(3,3,nel)
  IF nph GT 1 THEN phit=TRANSPOSE(phi) ELSE phit=phi
  IF nth GT 1 THEN thetat=TRANSPOSE(theta) ELSE thetat=theta
  IF nw  GT 1 THEN w0t=TRANSPOSE(w0) ELSE w0t=w0


  cp=COS(phi)
  ct=COS(theta)
  sp=SIN(phi)
  st=SIN(theta)
  sw=SIN(w0)
  cw=COS(w0)

  ematrix(0,0,*)=(cw*cp-sw*sp*ct)
  ematrix(1,0,*)=(cw*sp+sw*cp*ct)
  ematrix(2,0,*)=(sw*st)
  ematrix(0,1,*)=(-sw*cp-cw*sp*ct)
  ematrix(1,1,*)=(-sw*sp+cw*cp*ct)
  ematrix(2,1,*)=(cw*st)
  ematrix(0,2,*)=(sp*st)
  ematrix(1,2,*)=(-cp*st)
  ematrix(2,2,*)=(ct) 

  return,ematrix
  end
